#!/usr/bin/perl

#	  mei2tgd jim16 Mon 15 November 13:13:00
#
#	  Copyright (C) 2006 Jim16
#
#	  This is free software; you can redistribute it and/or modify
#	  it under the terms of the GNU General Public License as published by
#	  the Free Software Foundation; either version 2 of the License, or
#	  (at your option) any later version.
#
#	  The software is distributed in the hope that it will be useful,
#	  but WITHOUT ANY WARRANTY; without even the implied warranty of
#	  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	  GNU General Public License for more details.
#
#	  You should have received a copy of the GNU General Public License
#	  along with this software; if not, write to the Free Software
#	  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

my @tgd;
my $version = "0.1";
my $singlefile = 0;
my $output = "";
my %channel_rename;
my $channel_rename_string = "";
my %channels;
my @channel_temp;
my $channelname_temp;
my $log = 1;
my $logstring;
my $config;

&printOut("\t\t\t\tMEI 2 TGD $version\n\n");

# Get the switches from the terminal
foreach( @ARGV ){	# Go thru each argument sent to program
	if ( $_ =~ /^-o/i ){ 
		$output = $_; 
		$output =~ s/-o=//ig; 
		if( $output =~ /\/$/ ){	# Remove append slash if present
			$output =~ s/\/$//;
		}
	} elsif( $_ =~ /^-rename/ ){								# Set the delay betwwen grabbing pages
		$channel_rename_string = $_;
		$channel_rename_string =~ s/-rename=//ig;	
		&createChannelRename($channel_rename_string);
	} elsif( $_ =~ /^-nolog/ ){								# Set the delay betwwen grabbing pages
		$log = 1;
	} elsif( $_ =~ /^-config/ ){								# Set the delay betwwen grabbing pages
		$config = $_;
		$config =~ s/-config=//ig;	
	}
}

if( $config ){
	&printOut("Reading configuration file");
	open( FILE, "$output/$config" );
	while( <FILE> ){
		chomp;
		(my $key, my $value ) = split(/::/, $_ );
		if( $key =~ /file/i ){
			$xmlfile = $value;
		} elsif( $key =~ /rename/i ){
			$value =~ s/\"//g;
			$channel_rename_string = $value;
			&createChannelRename($value);
		}
	}
	close FILE;
	&printOut(" - Done\n\n");
}

# Exit and print errors if minimum details are not present
if( !$output ){
	!$output?&printOut("ERROR ::: No Output Directory -> -o=( Output Directory ) e.g. -o=output\n"):print "";
	exit;
}

# Print settings to be used in program to the console/terminal
&printOut("Settings to be used\n\n");
&printOut("     Output Dir  ::: $output\n");
if( $singlefile ){ &printOut("  Output File/s  ::: Single File\n"); }
else { &printOut("  Output File/s  ::: Seperate Files\n"); }
&printOut("Rename Channels  ::: $channel_rename_string\n");
if( $log ){ &printOut("            Log  ::: mei2tgd.log\n\n"); }
else { &printOut("            Log  ::: none\n\n"); }
&printOut("Starting conversion\n");


# Lets start :)
open( FILE, "$output/Freeview.mei" );
while( <FILE> ){
	chomp;
	my @mei_show = split( /\|/ );
	if( @mei_show > 0 ){
		my $show = {};	# Clear hash and begin new
		$show->{'date-time'} = &dateTimeFormat($mei_show[1]);
#		print "Date Time : ".$show->{'date-time'}."\n";
		$show->{'duration'} = &trim($mei_show[13]);
#		print "Show duration : ".$show->{'duration'}."\n";
		$show->{'channel'} = &trim($mei_show[3]);
#		print "Channel : ".$show->{'channel'}."\n";
		$show->{'title'} = &trim($mei_show[4]);
#		print "Title : ".$show->{'title'}."\n";
		$show->{'sub-title'} = &trim($mei_show[5]);
#		print "Subtitle : ".$show->{'sub-title'}."\n";
		if( &trim($mei_show[6]) =~ /TRUE/ ){
			$show->{'repeat'} = 1;
		}
		$show->{'desc'} = &trim($mei_show[11]);
#		print "Description : ".$show->{'desc'}."\n";
		if( $show->{'duration'} > 0 ){
			push(@tgd,$show);
		}
	}
	
}
close FILE;

&outputFiles();

# Write the log of our progress
if( $log ){
	open( LOG, ">$output/mei2tgd.log" );
	print LOG $logstring."\n";
	close FILE;
}

exit;

sub printOut(){
	my $text = shift @_;
	print $text;
	$logstring .= $text;
}

sub createChannelRename(){
	my $input = shift @_;
	my @rename = split(/:/,$input);
	foreach( @rename ){
		( my $original, my $newoutput ) = split( /-/ );
		$channel_rename{$original} = $newoutput;
	}
}

# function to split days and write them to files.
sub outputFiles(){
	print "\n";
	if( !$singlefile ){
		my @files;			# Stores the day filename to output
		foreach( @tgd ){	# Go thru and get a list of all the days
			my $fname = &filenameString($_->{'date-time'});
			my $filenameexists = 0;
			# Check to seee whether date exists in files array
			foreach( @files ){	
				if( $_ =~ $fname ){	
					$filenameexists = 1;
					end;
				}
			}
			if( !$filenameexists ){		# If the day hasn't been found then add to files array
				push(@files,$fname);	
			}
		}
		@files = sort( @files );
		foreach my $file ( @files ){
			my @shows;
			foreach my $tgd ( @tgd ){
				if( &filenameString($tgd->{'date-time'}) =~ /$file/ ){
					my $cat = "";
					if( $tgd->{'category'} ){
						$cat = " [ ".$tgd->{'category'}." ]";
					}
					my $actor = "";
					if( $tgd->{'actor'} ){
						$actor = " Cast: ".$tgd->{'actor'};
					}
					my $director = "";
					if( $tgd->{'director'} ){
						$director = " Director: ".$tgd->{'director'}.".";
					}
					my $year = "";
					if( $tgd->{'date'} ){
						$year = " Year: ".$tgd->{'date'}.".";
					}
					my $repeat = "";
					if( $tgd->{'repeat'} ){
						$repeat = " [Rpt]";
					}					
					push(@shows,"$tgd->{'channel'}\t$tgd->{'date-time'}\t$tgd->{'duration'}\t$tgd->{'title'}\t$tgd->{'sub-title'}$cat\t$tgd->{'desc'}$repeat$director$actor$year\t$tgd->{'rating'}\tN");
				}
			}
			@shows = sort ( @shows );
			open( OUTPUT, ">$output/$file.tgd" );
			&printOut("\tWriting $output/$file.tgd ( ".@shows." ) Shows\n");
			foreach my $show ( @shows ){
				print OUTPUT $show."\n";
			}
			close OUTPUT;
		}
	} else {
		# Output as a single file
		my @now_time = localtime(time);
		my $nowyear = sprintf('%02d', (@now_time[5]+1900));
		my $nowmonth = sprintf('%02d', (@now_time[4]+1));
		my $nowday = sprintf('%02d', (@now_time[3]));
		my $file = "$nowyear$nowmonth$nowday";
		my @shows;
		foreach my $tgd ( @tgd ){
			my $cat = "";
			if( $tgd->{'category'} ){
				$cat = " [ ".$tgd->{'category'}." ]";
			}
			my $actor = "";
			if( $tgd->{'actor'} ){
				$actor = " Cast: ".$tgd->{'actor'};
			}
			my $director = "";
			if( $tgd->{'director'} ){
				$director = " Director: ".$tgd->{'director'}.".";
			}
			my $year = "";
			if( $tgd->{'date'} ){
				$year = " Year: ".$tgd->{'date'}.".";
			}				
			my $repeat = "";
			if( $tgd->{'repeat'} ){
				$repeat = " [Rpt]";
			}					
			push(@shows,"$tgd->{'channel'}\t$tgd->{'date-time'}\t$tgd->{'duration'}\t$tgd->{'title'}\t$tgd->{'sub-title'}$cat\t$tgd->{'desc'}$repeat$director$actor$year\t$tgd->{'rating'}\tN");
		}
		@shows = sort ( @shows );
		open( OUTPUT, ">$output/$file.tgd" );
		foreach my $show ( @shows ){
			print OUTPUT $show."\n";
		}
		close OUTPUT;
	}
	&printOut("\n");
}

# function that creates a string YYYYMMDD from DateTime String.
sub filenameString(){
	my $value = substr(shift @_,0,10);	# Only get the YYYY-MM-DD part of the DateTime String
	$value =~ s/-//g;	# Remove all - from the string
	return $value;
}

# function to create the Date Time string for the tgd elements ( YYYY-MM-DD HH:MM )
sub dateTimeFormat(){
	my $time = shift @_;
	my $year = substr($time,0,4);
	my $month = substr($time,4,2);
	my $date = substr($time,6,2);
	my $hour = substr($time,8,2);
	my $min = substr($time,10,2);
	if( $hour > 23 ){ 
		$hour = sprintf("%02d",($hour - 24)); 
		@dates = &incrementDate($year,$month,$date);
		$date = sprintf("%02d",@dates[2]);
		$month = sprintf("%02d",@dates[1]);
		$year = sprintf("%04d",@dates[0]);
	}
	$hour = sprintf("%02d",$hour);
	$min = sprintf("%02d",$min);
	return "$year-$month-$date $hour:$min";
}

sub trim(){
	my $string = shift;
	$string =~ s/^\s+//;
	$string =~ s/\s+$//;
	return $string;
}

